'use strict';

// @importFmJsLib "Results.js"

function AgentUptimeCheck() {
    this.wmiNamespace = "\\\\.\\root\\cimv2";
    this.SYSTEM_START_EVENT_ID = 6005; //12 might also work
    this.SYSTEM_STOP_EVENT_ID = 6006; //13 might also work
    this.RUNNING = "running"; //13 might also work
    this.STOPPED = "stopped"; //13 might also work
    this.response = {
        info: "",
        result: "",
        data: {}
    };
}

AgentUptimeCheck.prototype.run = function(config) {
    
    var apiCall = WMI_API.ConnectServer(this.wmiNamespace);
    if( !apiCall) {
        this.response.data.errorCode = "no connection";
        return this.response;
    }

    var now = new Date();
    var nowGMT = new Date(now.valueOf() + ( now.getTimezoneOffset() * 60 * 1000));

    this.generateFullResponse(nowGMT);

    
    return this.response;
};

AgentUptimeCheck.prototype.twentyFourHoursAgo = function(now) {
    return new Date(now - 24*60*60*1000);
}

AgentUptimeCheck.prototype.twentyFourHoursAgoString = function(now) {
    var twentyFourHoursAgo = this.twentyFourHoursAgo(now);
    var twentyFourHoursAgoString = twentyFourHoursAgo.toISOString();
    // 2015-06-30T12:00:59.123Z
    // should be 20150630120059.123000-000, look at Microsoft documentation...
    twentyFourHoursAgoString = twentyFourHoursAgoString.split(/[^\d\.]/).join("") + "000-000";
    return twentyFourHoursAgoString;
}

AgentUptimeCheck.prototype.buildQueryAgent = function(now) {
    var query = "SELECT RecordNumber, TimeGenerated, EventCode, Type, Category, SourceName, Message FROM Win32_NTLogEvent WHERE EventCode = 7036 and (Message = 'The Advanced Monitoring Agent service entered the running state.' or Message = 'The Advanced Monitoring Agent service entered the stopped state.')";

    query += " AND TimeGenerated >= '" + this.twentyFourHoursAgoString(now) + "'";

    return query;
}

AgentUptimeCheck.prototype.buildQuerySystem = function(now) {
    var query = "SELECT RecordNumber, TimeGenerated, EventCode, Type, Category, SourceName, Message FROM Win32_NTLogEvent WHERE ( EventCode = " + this.SYSTEM_START_EVENT_ID + " OR EventCode = " + this.SYSTEM_STOP_EVENT_ID + " ) "; 
    query += " AND TimeGenerated >= '" + this.twentyFourHoursAgoString(now) + "'";

    return query;
}

AgentUptimeCheck.prototype.generateFullResponse = function(now) {
    this.generateAgentResponse(now);
    this.generateSystemResponse(now);

    this.response.data.percentageSystemUptimeOf24Hours = Math.round( (this.response.data.systemUptimeInSeconds / (24*60*60)) * 100 * 100 ) / 100;
    this.response.data.percentageAgentUptimeOf24Hours = Math.round( (this.response.data.agentUptimeInSeconds / (24*60*60)) * 100 * 100 ) / 100;
    this.response.data.percentageAgentUptimeOfSystemUptime = Math.round( this.response.data.agentUptimeInSeconds / this.response.data.systemUptimeInSeconds * 100 * 100 ) / 100;
    this.response.result = results.PASSING;

    return this.response;
}

AgentUptimeCheck.prototype.generateAgentResponse = function(now) {
    var query = this.buildQueryAgent(now);
    var queryResults = WMI_API.ExecQuery(query);
    
    var starts = 0;
    var stops = 0;
    var unexpectedStarts = 0;
    var unexpectedStops = 0;
    var uptime = 0;

    if( !this.isEmpty(queryResults) ){
        var length = Object.keys(queryResults).length;
        var previousEvent = "";
        var previousEventTime = 0;
        for (var i = 0; i < length; i++ ) {
            var currentRecord = queryResults[i];

            //First record (last chronologically)
            if(i==0) {
                //first returned is most recent if this is a start then add now to uptime
                if(currentRecord.Message.indexOf(this.RUNNING) != -1) {
                    uptime += now.getTime() / 1000;
                }
            }

            if(currentRecord.Message.indexOf(this.STOPPED) != -1) {
                stops++;
                if(previousEvent != this.STOPPED) {
                    uptime -= previousEventTime;
                    previousEventTime = this.parseCimDateTime(currentRecord.TimeGenerated);
                    previousEvent = this.STOPPED;
                } else {
                    unexpectedStops++;
                    previousEventTime = this.parseCimDateTime(currentRecord.TimeGenerated);
                }
            }
            if(currentRecord.Message.indexOf(this.RUNNING) != -1) {
                starts++;
                if(previousEvent != this.RUNNING) {
                    uptime += previousEventTime;
                    previousEventTime = this.parseCimDateTime(currentRecord.TimeGenerated);
                    previousEvent = this.RUNNING;
                } else {
                    unexpectedStarts++;
                    previousEventTime = this.parseCimDateTime(currentRecord.TimeGenerated);
                }
            }

            //Last record (first chronologically)
            if(i==(length-1)) {
                //last returned is furthest away that's not over 24 hours ago if so subtract twenty four hours ago from uptime
                if(currentRecord.Message.indexOf(this.STOPPED) != -1) {
                    uptime -= this.twentyFourHoursAgo(now).getTime() / 1000;
                }
                if(previousEvent != this.RUNNING) {
                    uptime += previousEventTime;
                } else {
                    uptime -= previousEventTime;
                }
            }
        }
    } else {
        uptime = 24*60*60; //agent running now so has been for last 24 hours.
    }
    
    this.response.data.agentStarts = starts;
    this.response.data.agentStops = stops;
    this.response.data.agentUnexpectedStarts = unexpectedStarts;
    this.response.data.agentUnexpectedStops = unexpectedStops;
    this.response.data.agentUptimeInSeconds = uptime;
    return this.response;
}

AgentUptimeCheck.prototype.generateSystemResponse = function(now) {
    var query = this.buildQuerySystem(now);
    var queryResults = WMI_API.ExecQuery(query);
    
    var starts = 0;
    var stops = 0;
    var unexpectedStarts = 0;
    var unexpectedStops = 0;
    var uptime = 0;
    
    if( !this.isEmpty(queryResults) ){
        var length = Object.keys(queryResults).length;
        var previousEvent = "";
        var previousEventTime = 0;
        for (var i = 0; i < length; i++ ) {
            var currentRecord = queryResults[i];
            
            //First record (last chronologically)
            if(i==0) {
                //first returned is most recent if this is a start then add now to uptime
                if(currentRecord.EventCode == this.SYSTEM_START_EVENT_ID) {
                    uptime += now.getTime() / 1000;
                }
            }
            
            if(currentRecord.EventCode == this.SYSTEM_STOP_EVENT_ID) {
                stops++;
                if(previousEvent != this.STOPPED) {
                    uptime -= previousEventTime;
                    previousEventTime = this.parseCimDateTime(currentRecord.TimeGenerated);
                    previousEvent = this.STOPPED;
                } else {
                    unexpectedStops++;
                    previousEventTime = this.parseCimDateTime(currentRecord.TimeGenerated);
                }
            }
            if(currentRecord.EventCode == this.SYSTEM_START_EVENT_ID) {
                starts++;
                if(previousEvent != this.RUNNING) {
                    uptime += previousEventTime;
                    previousEventTime = this.parseCimDateTime(currentRecord.TimeGenerated);
                    previousEvent = this.RUNNING;
                } else {
                    unexpectedStarts++;
                    previousEventTime = this.parseCimDateTime(currentRecord.TimeGenerated);
                }
            }

            if(i==(length-1)) {
                //last returned is furthest away that's not over 24 hours ago if so subtract twenty four hours ago from uptime
                if(currentRecord.EventCode == this.SYSTEM_STOP_EVENT_ID) {
                    uptime -= this.twentyFourHoursAgo(now).getTime() / 1000;
                }
                if(previousEvent != this.RUNNING) {
                    uptime += previousEventTime;
                } else {
                    uptime -= previousEventTime;
                }
            }
        }
    } else {
        uptime = 24*60*60; //agent running now so has been for last 14 hours.
    }
    
    this.response.data.systemStarts = starts;
    this.response.data.systemStops = stops;
    this.response.data.systemUnexpectedStarts = unexpectedStarts;
    this.response.data.systemUnexpectedStops = unexpectedStops;
    this.response.data.systemUptimeInSeconds = uptime;
    return this.response;
}


AgentUptimeCheck.prototype.isEmpty = function(queryResults) {
    if(queryResults === undefined || 0 == Object.keys(queryResults).length) {
        return true;
    }

    return false;
}

//CIM format: yyyymmddHHMMSS.mmmmmmsUUU
//Timestamp: seconds since 1970 (localised)
// Some code in common with critical event check, should be extracted
AgentUptimeCheck.prototype.parseCimDateTime = function(cimDateTime) {
    if (cimDateTime === undefined) {
        return undefined;
    }

    var dateParts = cimDateTime.match(/(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})\.(\d{3})(\d{3})(\-|\+)(\d{3})/);
    dateParts.splice(0, 1);

    var timeZone = dateParts[8] + dateParts[9];

    var minutesOffset = parseInt(timeZone);

    if (minutesOffset == 0) {
        timeZone = "Z";
    } else {
        var sign = (minutesOffset > 0 ? "+" : "-");
        minutesOffset = Math.abs(minutesOffset);
        timeZone = sign + ("00" + parseInt(minutesOffset / 60)).slice(-2) + ":" + ("00" + minutesOffset % 60).slice(-2);
    }

    var date = dateParts[0] + "-" + dateParts[1] + "-" + dateParts[2] + "T" + dateParts[3] + ":" + dateParts[4] + ":" + dateParts[5] + "." + dateParts[6] + timeZone;
    var dateObject = new Date(dateParts[0], dateParts[1] - 1, dateParts[2], dateParts[3], dateParts[4], dateParts[5], dateParts[6]);
    var dateAsUnixTimestamp = Math.round(dateObject.getTime() / 1000);
    return dateAsUnixTimestamp;
}


function getCheck() {
    return new AgentUptimeCheck();
}
/*CHECKSUM: 
173efecfab64f42b9511f7171f17a8496f94b75b *C:\jenkins\workspace\FmPlugin_release_1.5.19\build\outputs\mainapp\RelWithDebInfo\Checks\agent_uptime.js
*/ 
