'use strict';

// @importFmJsLib "Results.js"
// @importFmJsLib "Errors.js"

function BandwidthCheck() {
    this.responses = [];

    this.firstRunResults = {
        Timestamp: "0",
        ReceivedBytes: "0",
        TransmittedBytes: "0"
    };

    this.OID_IF_PORT_SPEED = "1.3.6.1.2.1.2.2.1.5.";
    this.OID_IF_PORT_HI_RX_OCTETS = "1.3.6.1.2.1.31.1.1.1.6.";
    this.OID_IF_PORT_HI_TX_OCTETS = "1.3.6.1.2.1.31.1.1.1.10.";
    this.OID_IF_PORT_RX_OCTETS = "1.3.6.1.2.1.2.2.1.10.";
    this.OID_IF_PORT_TX_OCTETS = "1.3.6.1.2.1.2.2.1.16.";

    this.SNMPv1 = 1;
    this.SNMPv2c = 2;

    this.sNMP_SYNTAX_CNTR32 = 65;
    this.sNMP_SYNTAX_GAUGE32 = 66;
    this.sNMP_SYNTAX_CNTR64 = 70;

    this.haveServerConfig = false;

    this.ERROR_OID_DOES_NOT_EXIST = "SNMP: OID does not exist";
}

BandwidthCheck.prototype.run = function(configList) {
    var serverConfig = {};
    var oidsArray = [];

    for (var checkId in configList) {
        var config = configList[checkId];
        if (!this.haveServerConfig) {
            serverConfig = this.getServerConfig(config);
        }
        // we are pushing the content of another array, so a simple push(array) does not work
        // we need to do push.apply(array).
        oidsArray.push.apply(oidsArray, this.getOids(config));
    }

    var apiResponse = SNMPAPI.snmpRequest(oidsArray, serverConfig);

    if (apiResponse.success) {
        this.generateSuccessfulResponses(configList, apiResponse);
    } else {
        this.generateFailedResponses(configList, apiResponse.message);
    }
    return this.responses;
}

BandwidthCheck.prototype.isGigabit = function(config) {
    return config.hispeed == 1;
}

BandwidthCheck.prototype.getServerConfig = function(config) {
    var version = this.isGigabit(config) ? this.SNMPv2c : this.SNMPv1;
    var serverConfig = {
        host: config.host,
        port: config.port,
        community: config.community,
        version: version
    };
    this.haveServerConfig = true;
    return serverConfig;
}

BandwidthCheck.prototype.generateFailedResponses = function(configList, errorString) {
    for (var checkId in configList) {
        var response = this.generateSingleFailedResponse(checkId, errorString);
        this.responses.push(response);
    }
}

BandwidthCheck.prototype.generateSingleFailedResponse = function(checkId, errorString) {
    var response = {
        checkId: checkId,
        result: results.FAILING,
        data: {
            "errorCode": errors[errorString],
            "Output": errorString
        }
    };
    return response;
}

BandwidthCheck.prototype.generateSuccessfulResponses = function(configList, apiResponse) {
    for (var checkId in configList) {
        var config = configList[checkId];
        var lastConfig = FileSystemAPI.getLastCheckConfig(checkId);
        var lastResult = {};
        if (this.isSameConfig(config, lastConfig)) {
            lastResult = this.getPreviousResult(checkId);
        }
        var currentValues = this.getDataFromApiResponse(apiResponse.results, apiResponse.resultsSyntax, this.getOids(config));
        if (currentValues.success) {
            this.responses.push(this.buildResponse(checkId, currentValues, lastResult, config.threshold_rx, config.threshold_tx, config.units_rx, config.units_tx));
        } else {
            this.responses.push(this.generateSingleFailedResponse(checkId, this.ERROR_OID_DOES_NOT_EXIST));
        }
    }
}

BandwidthCheck.prototype.isSameConfig = function(config, lastConfig) {
    return config.host == lastConfig.host &&
        config.port == lastConfig.port &&
        config.community == lastConfig.community &&
        config.hispeed == lastConfig.hispeed &&
        config.routerport == lastConfig.routerport;
}

BandwidthCheck.prototype.buildResponse = function(checkId, currentValues, lastResult, threshold_rx, threshold_tx, units_rx, units_tx) {
    var response = {
        checkId: checkId,
        result: "",
        data: {}
    };

    // If this port has no previous data (usually because it's a first run) then
    // set the values to zero so that first run does not produce odd results.
    var receivedBytesPerSecond = 0;
    var transmittedBytesPerSecond = 0;
    var receivedKilobitsPerSecond = 0;
    var transmittedKilobitsPerSecond = 0;
    var sampleRate = 0;

    if (lastResult.Timestamp && lastResult.Timestamp != 0) {
        receivedBytesPerSecond = BandwidthAPI.getTransmittedBytesPerSecond(currentValues.ReceivedBytes, lastResult.ReceivedBytes, currentValues.Timestamp, lastResult.Timestamp);
        transmittedBytesPerSecond = BandwidthAPI.getTransmittedBytesPerSecond(currentValues.TransmittedBytes, lastResult.TransmittedBytes, currentValues.Timestamp, lastResult.Timestamp);

        receivedBytesPerSecond = receivedBytesPerSecond.toFixed(3);
        transmittedBytesPerSecond = transmittedBytesPerSecond.toFixed(3);

        receivedKilobitsPerSecond = ((receivedBytesPerSecond * 8) / 1000).toFixed(3);
        transmittedKilobitsPerSecond = ((transmittedBytesPerSecond * 8) / 1000).toFixed(3);

        sampleRate = currentValues.Timestamp - lastResult.Timestamp;

        var thresholdReceivedInBitsPerSecond = this.getThreshold(threshold_rx, units_rx);

        var thresholdTransmittedInBitsPerSecond = this.getThreshold(threshold_tx, units_tx);

        var receivedBitsPerSecond = receivedBytesPerSecond * 8;
        var transmittedBitsPerSecond = transmittedBytesPerSecond * 8;

        if (receivedBitsPerSecond > thresholdReceivedInBitsPerSecond || transmittedBitsPerSecond > thresholdTransmittedInBitsPerSecond) {
            response.result = results.FAILING;
        } else {
            response.result = results.PASSING;
        }
    } else {
        response.result = results.PASSING;
    }

    response.data.SampleRate = "" + sampleRate;
    response.data.PortSpeedKBPerSecond = "" + currentValues.PortSpeedKBPerSecond;
    response.data.ReceivedKBPerSecond = "" + receivedKilobitsPerSecond;
    response.data.TransmittedKBPerSecond = "" + transmittedKilobitsPerSecond;
    response.data.ReceivedBytes = "" + currentValues.ReceivedBytes;
    response.data.TransmittedBytes = "" + currentValues.TransmittedBytes;
    response.data.Timestamp = "" + currentValues.Timestamp;
    return response;
}

BandwidthCheck.prototype.getThreshold = function(threshold, units) {
    if (units != 0) {
        return this.getThresholdInMBitsPerSec(threshold);
    } else {
        return this.getThresholdInKBitsPerSec(threshold);
    }
}

BandwidthCheck.prototype.getThresholdInMBitsPerSec = function(threshold) {
    return threshold * 1000000;
}

BandwidthCheck.prototype.getThresholdInKBitsPerSec = function(threshold) {
    return threshold * 1000;
}

BandwidthCheck.prototype.getDataFromApiResponse = function(results, resultsSyntax, oidsArray) {
    var currentValues = {};

    currentValues.success = true;

    for (var i = 0; i < oidsArray.length; i++) {
        if (!this.isValidSyntax(resultsSyntax[oidsArray[i]])) {
            currentValues.success = false;
            return currentValues;
        }
        if (this.contains(oidsArray[i], this.OID_IF_PORT_SPEED)) {
            currentValues.PortSpeedKBPerSecond = results[oidsArray[i]] / 1000;
        } else {
            if (this.contains(oidsArray[i], this.OID_IF_PORT_HI_RX_OCTETS) || this.contains(oidsArray[i], this.OID_IF_PORT_RX_OCTETS)) {
                currentValues.ReceivedBytes = parseInt(results[oidsArray[i]]); //parseInt will also convert from hex if starting 0x
            } else {
                if (this.contains(oidsArray[i], this.OID_IF_PORT_HI_TX_OCTETS) || this.contains(oidsArray[i], this.OID_IF_PORT_TX_OCTETS)) {
                    currentValues.TransmittedBytes = parseInt(results[oidsArray[i]]); //parseInt will also convert from hex if starting 0x
                }
            }
        }
    }

    // timestamp in seconds
    currentValues.Timestamp = Math.floor(Date.now() / 1000);

    return currentValues;
}

BandwidthCheck.prototype.isValidSyntax = function(syntax) {
    return syntax == this.sNMP_SYNTAX_GAUGE32 ||
        syntax == this.sNMP_SYNTAX_CNTR32 ||
        syntax == this.sNMP_SYNTAX_CNTR64;
}

BandwidthCheck.prototype.contains = function(inputString, stringToFind) {
    return inputString.indexOf(stringToFind) > -1;
}

BandwidthCheck.prototype.getOids = function(config) {
    var oids = [];

    oids.push(this.OID_IF_PORT_SPEED + config.routerport);

    if (this.isGigabit(config)) {
        oids.push(this.OID_IF_PORT_HI_RX_OCTETS + config.routerport);
        oids.push(this.OID_IF_PORT_HI_TX_OCTETS + config.routerport);
    } else {
        oids.push(this.OID_IF_PORT_RX_OCTETS + config.routerport);
        oids.push(this.OID_IF_PORT_TX_OCTETS + config.routerport);
    }

    return oids;
}

BandwidthCheck.prototype.getPreviousResult = function(checkId) {
    var previousResult = FileSystemAPI.getLastCheckResult(checkId);

    if (previousResult.isEmpty || (previousResult.errorCode && previousResult.errorCode != "")) {
        // either no previous result, or previous result without errorCode
        return this.firstRunResults;
    }

    return previousResult;
}


function getCheck() {
    return new BandwidthCheck();
}/*CHECKSUM: 
fff106359b4b31160ffc63d2a8da58fce8e1d896 *C:\jenkins\workspace\FmPlugin_release_1.5.19\build\outputs\mainapp\RelWithDebInfo\Checks\check_bandwidth.js
*/ 
