'use strict';

// @importFmJsLib "CimDate.js"
// @importFmJsLib "Results.js"
// @importFmJsLib "Errors.js"

function EventLogCheck() {
	this.checkConfig;
	this.WMINAMESPACE = "\\\\.\\root\\cimv2";
	this.response = {
		info: "",
		result: "",
		data: {}
	};

	this.cimDateUtils = new CimDate();
}

// if the elements of the two arrays are the same but in different order
// this method will return false. This is because the exlusions in the config
// is an array of objects, and we can not sort it easily. So if the client
// edit the exclusion list the check will use the interval instead of the last run time
EventLogCheck.prototype.compareArrays = function(firstArray, secondArray, isExclusions) {
	// if the second array is a falsy value, return
	if (!secondArray)
		return false;

	// compare lengths - can save a lot of time
	if (firstArray.length != secondArray.length)
		return false;

	for (var i = 0, l = firstArray.length; i < l; i++) {
		if (isExclusions) {
			// compare exclusions
			if (firstArray[i].eventid != secondArray[i].eventid || firstArray[i].eventsource != secondArray[i].eventsource) {
				return false;
			}
		} else if (firstArray[i] != secondArray[i]) {
			// Warning - two different object instances will never be equal: {x:20} != {x:20}
			return false;
		}
	}
	return true;
}


EventLogCheck.prototype.run = function(config) {
	this.checkConfig = config;
	this.response.result = results.FAILING;

	var alertIfFound = this.checkConfig.alertwhen == "1";
	var alertIfNotFound = this.checkConfig.alertwhen == "0";

	var apiCall = WMI_API.ConnectServer(this.WMINAMESPACE);
	if (!apiCall) {
		this.response.data.errorCode = errors.ERROR_WMI_CANNOT_CONNECT;
		return this.response;
	}

	var query = this.buildQuery(new Date());

	var queryResults = WMI_API.ExecQuery(query);
	if (this.checkConfig.exclusions != "") {
		this.removeExclusions(queryResults);
	}

	if (this.checkConfig.applyexclusions == "1") {
		this.removeCriticalExclusions(queryResults);
	}

	var numRecords = 0;

	if (!this.returnedEventsEmpty(queryResults)) {
		numRecords = queryResults.length;
	}

	if (numRecords == 0 && alertIfFound) {
		this.response.result = results.PASSING;
	}
	if (numRecords > 0 && alertIfNotFound) {
		this.response.result = results.PASSING;
	}

	//TODO remove before live?
	this.response.data.Query = query;

	if (numRecords > 0) {
		this.response.data.EventFound = "1";
		var firstResult = queryResults[0]; //newest are first
		this.response.data.EventTime = this.cimDateUtils.parseCimDateTimeToTimestamp(firstResult.TimeGenerated);
		this.response.data.ID = firstResult.EventCode;
		this.response.data.Type = firstResult.Type;
		this.response.data.Source = firstResult.SourceName;
		this.response.data.Message = firstResult.Message;

	} else {
		this.response.data.EventFound = "0";
	}

	return this.response;
}

EventLogCheck.prototype.buildQuery = function(now) {
	var query = "SELECT RecordNumber, TimeGenerated, EventCode, Type, Category, SourceName, Message FROM Win32_NTLogEvent WHERE";
	query += this.getLogFileQueryPart();
	query += " AND";
	query += this.getSourceNameQueryPart();

	var eventIDPart = this.getEventIDQueryPart();
	if (eventIDPart != "") {
		query += " AND";
		query += eventIDPart;
	}
	query += this.getTypeQueryPart();
	if (this.checkConfig.msgcontains == "1") {
		query += " AND";
		query += this.getContainsStringQueryPart();
	}
	query += this.getDateRangeQueryPart(now);
	return query;
}

EventLogCheck.prototype.getLogFileQueryPart = function() {
	return " Logfile = \"" + this.checkConfig.eventlog + "\"";
}

EventLogCheck.prototype.getSourceNameQueryPart = function() {
	var sourceName = this.checkConfig.source;
	var operator = "=";
	if (sourceName.indexOf("?") > -1) {
		sourceName = sourceName.replace(/\?/g, "_");
		operator = "LIKE";
	}
	if (sourceName.indexOf("*") > -1) {
		sourceName = sourceName.replace(/\*/g, "%");
		operator = "LIKE";
	}
	if (sourceName.indexOf("[") > -1) {
		sourceName = sourceName.replace(/\[/g, "[[]");
		operator = "LIKE";
	}
	sourceName = sourceName.replace(/\"/g, "\\\"");
	var queryPart = " SourceName " + operator + ' "' + sourceName + '"';
	return queryPart;
}

EventLogCheck.prototype.getContainsStringQueryPart = function() {
	var containsString = this.checkConfig.string;
	containsString = containsString.replace(/\?/g, "_");
	containsString = containsString.replace(/\*/g, "%");
	containsString = containsString.replace(/\[/g, "[[]");
	containsString = containsString.replace(/\"/g, "\\\"");
	return " Message LIKE \"%" + containsString + "%\"";
}

EventLogCheck.prototype.getEventIDQueryPart = function() {

	if (typeof this.checkConfig.eventids === "undefined" || this.checkConfig.eventids === null || this.checkConfig.eventids === "") {
		return '';
	}
	var eventIDs = [];
	if (this.checkConfig.eventids.indexOf(",") == -1) {
		eventIDs.push(this.checkConfig.eventids);
	} else {
		var eventIDs = this.checkConfig.eventids.split(',');
	}

	var first = true;
	var queryPart = " ( EventCode = ";
	var querySegment = " EventCode = ";
	var id;

	for (var i = 0; i < eventIDs.length; i++) {
		id = eventIDs[i];
		if (id == "*") {
			return '';
		}
		if (first == false) {
			queryPart += " OR";
			queryPart += querySegment;
		}
		queryPart += id.trim();
		first = false;

	}
	queryPart += " )"
	return queryPart;
}

EventLogCheck.prototype.getTypeQueryPart = function() {
	if (this.checkConfig.eventtypes.length == 0) {
		return '';
	}

	// Event types take from https://msdn.microsoft.com/en-us/library/aa394226(v=vs.85).aspx
	var eventTypeMapping = {
		'Success': 0,
		'Error': 1,
		'Warning': 2,
		'Information': 3,
		'Audit Success': 4,
		'Audit Failure': 5,

		// Artificial type name (see RM-7891)
		'Success None': 0
	};

	var eventTypeClauses = [];
	for (var i = 0; i < this.checkConfig.eventtypes.length; i++) {
		var eventType = this.checkConfig.eventtypes[i];
		if(eventTypeMapping.hasOwnProperty(eventType)) {
			var eventTypeId = eventTypeMapping[eventType];
			eventTypeClauses.push('EventType = ' + eventTypeId);
		}
	}

	if(eventTypeClauses.length === 0) {
		return '';
	}

	return ' AND ( ' + eventTypeClauses.join(' OR ') + ' )';
}

EventLogCheck.prototype.hasConfigChanged = function() {
	var lastConfig = FileSystemAPI.getLastCheckConfig(this.checkConfig.checkid);
	var result = this.checkConfig.alertwhen == lastConfig.alertwhen &&
		this.checkConfig.applyexclusions == lastConfig.applyexclusions &&
		this.checkConfig.eventids == lastConfig.eventids &&
		this.checkConfig.eventlog == lastConfig.eventlog &&
		this.checkConfig.msgcontains == lastConfig.msgcontains &&
		this.checkConfig.source == lastConfig.source &&
		this.checkConfig.string == lastConfig.string &&
		this.compareArrays(this.checkConfig.eventtypes, lastConfig.eventtypes, false) &&
		this.compareArrays(this.checkConfig.exclusions, lastConfig.exclusions, true);

	return !result;
}

EventLogCheck.prototype.getDateRangeQueryPart = function(now) {
	var lastExecutedAt = "";

	if (!this.hasConfigChanged()) {
		lastExecutedAt = FileSystemAPI.getLastCheckExecutedAt(this.checkConfig.checkid);
	}

	var intervalInMinutes = this.checkConfig.checkinterval;
	var fromDateTimeString;

	// The reason this is 23 hours is daylight saving time - this is the min length of a day,
	// and is what the DSC interval will be on a day where the clocks previously went forward.
	if (intervalInMinutes >= 23 * 60) { //e.g. old DSC
		fromDateTimeString = this.getCIMDateFromNowAndInterval(now, intervalInMinutes);
	} else { //e.g. old 247
		if (lastExecutedAt === "") { //first run or config changed
			fromDateTimeString = this.getCIMDateFromNowAndInterval(now, intervalInMinutes);
		} else {
			fromDateTimeString = this.convertCppISODateStringToCIM(lastExecutedAt);
		}
	}
	return " AND TimeGenerated >= '" + fromDateTimeString + "'";
}

EventLogCheck.prototype.getCIMDateFromNowAndInterval = function(now, intervalInMinutes) {
	var nowMinusInterval = new Date(now - (intervalInMinutes * 60 * 1000));
	return this.convertJavascriptISODateStringToCIM(nowMinusInterval.toISOString());
}

EventLogCheck.prototype.convertJavascriptISODateStringToCIM = function(dateIn) {
	return dateIn.split(/[^\d\.]/).join("") + "000-000";
}

EventLogCheck.prototype.convertCppISODateStringToCIM = function(dateIn) {
	var timezoneString = "-000";
	var dateInString = dateIn;

	if (dateIn.slice(-6).indexOf("-") > -1) {
		dateInString = dateIn.substring(0, dateIn.lastIndexOf("-"));
		var timezone = dateIn.substring(dateIn.lastIndexOf("-") + 1);
		var timezoneParts = timezone.split(":");
		var hoursBehind = parseInt(timezoneParts[0]) * 60 + parseInt(timezoneParts[1]);
		timezoneString = "-" + ("000".substring(hoursBehind.toString().length)) + hoursBehind.toString();
	}
	if (dateIn.slice(-6).indexOf("+") > -1) {
		dateInString = dateIn.substring(0, dateIn.lastIndexOf("+"));
		var timezone = dateIn.substring(dateIn.lastIndexOf("+"));
		var timezoneParts = timezone.split(":");
		var hoursAhead = parseInt(timezoneParts[0]) * 60 + parseInt(timezoneParts[1]);
		timezoneString = "+" + ("000".substring(hoursAhead.toString().length)) + hoursAhead.toString();
	}

	return dateInString.split(/[^\d\.]/).join("") + ".000000" + timezoneString;
}

EventLogCheck.prototype.removeExclusions = function(recordset) {

	var recordCount = recordset.length;
	var exclusionsJSON = this.checkConfig.exclusions;
	var exclusionCount = Object.keys(exclusionsJSON).length;

	for (var j = exclusionCount - 1; j >= 0; j--) {

		var exclusion = exclusionsJSON[j];
		for (var i = recordCount - 1; i >= 0; i--) {

			var record = recordset[i];
			if (!record || typeof record == "undefined") {
				continue;
			}

			if (this.recordShouldBeRemoved(record, exclusion.eventid, exclusion.eventsource)) {
				recordset.splice(i, 1);
			}

		}
	}
}

EventLogCheck.prototype.removeCriticalExclusions = function(recordset) {
	var exclusions = ExcludedEventsAPI.getExcludedCriticalEvents();
	var items = recordset.length;
	for (var i = items - 1; i >= 0; i--) {
		var record = recordset[i];
		var exclusion;
		var eventidIndex;
		if (!record || typeof record == "undefined") {
			continue;
		}

		for (exclusion in exclusions) {
			var eventids = exclusions[exclusion];
			for (eventidIndex in eventids) {
				var eventid = eventids[eventidIndex];
				if (this.recordShouldBeRemoved(record, eventid, exclusion)) {
					recordset.splice(i, 1);
				}
			}
		}
	}
}

EventLogCheck.prototype.returnedEventsEmpty = function(queryResults) {
	if (queryResults.length <= 1) {
		if (!queryResults[0] || !queryResults[0].SourceName) {
			return true;
		}
	}
	return false;

}

EventLogCheck.prototype.recordShouldBeRemoved = function(record, eventid, source) {
	if (eventid == record['EventCode'] || eventid == "*" || eventid == ""|| eventid == "_ALL") {
		var exclusionRegex = new RegExp(this.toRegex(source), 'i');
		if (record.SourceName.match(exclusionRegex)) {
			return true;
		}
	}
	return false;
}

EventLogCheck.prototype.toRegex = function(unformatted) {

	var formatted = unformatted;
	if (formatted == "") {
		formatted = "*";
	}
	if (formatted == "___MATCH___ALL___SOURCES___") {
		formatted = "*";
	}
	formatted = formatted.replace(/([()[{+.$^\\|])/g, '\\$1');
	formatted = formatted.replace(/\*/g, '.*');
	formatted = formatted.replace(/\?/g, '.');
	formatted = formatted = '^' + formatted + '$';
	return formatted;
}

function getCheck() {
	return new EventLogCheck();
}
/*CHECKSUM: 
d17a4c87011fc561c754e2605d34a21faa6a311c *C:\jenkins\workspace\FmPlugin_release_1.5.19\build\outputs\mainapp\RelWithDebInfo\Checks\check_event_log.js
*/ 
