'use strict';

// @importFmJsLib "CimDate.js"
// @importFmJsLib "Results.js"
// @importFmJsLib "Errors.js"

function HackerCheck() {
    this.wmiNamespace = "\\\\.\\root\\cimv2";
    this.WMI_INSERTION_STRING_SOURCE_INDEX_FOR_4625 = 19;
    this.WMI_INSERTION_STRING_USERNAME_INDEX_FOR_4625 = 5;
    this.WMI_INSERTION_STRING_USERNAME_INDEX_FOR_4776 = 1;
    this.WMI_INSERTION_STRING_SOURCE_INDEX_FOR_4776 = 2;
    this.WMI_INSERTION_STRING_USERNAME_INDEX_FOR_4768 = 0;
    this.WMI_INSERTION_STRING_SOURCE_INDEX_FOR_4768 = 3;
    this.WMI_INSERTION_STRING_USERNAME_INDEX_FOR_4771 = 0;
    this.WMI_INSERTION_STRING_SOURCE_INDEX_FOR_4771 = 2;
    this.WMI_INSERTION_STRING_USERNAME_INDEX_FOR_4772 = 0;
    this.WMI_INSERTION_STRING_SOURCE_INDEX_FOR_4772 = 5;
    this.WMI_INSERTION_STRING_USERNAME_INDEX_FOR_4777 = 1;
    this.WMI_INSERTION_STRING_SOURCE_INDEX_FOR_4777 = 2;

    this.response = {
        info: "",
        result: "",
        data: {
        }
    };

    this.cimDateUtils = new CimDate();
}

HackerCheck.prototype.run = function(config) {
    this.response.result = results.PASSING;

    var apiCall = WMI_API.ConnectServer(this.wmiNamespace);
    if (!apiCall) {
        this.response.data.errorCode = errors.ERROR_WMI_CANNOT_CONNECT;
        this.response.result = results.FAILING;
        return this.response;
    }
    this.response.data.query = this.getQuery();
    var queryResults = WMI_API.ExecQuery(this.response.data.query);
    this.response.data.count = 0;
    if (!this.returnedEventsEmpty(queryResults)) {
        this.response.data.count = Object.keys(queryResults).length;
        this.response.data.events = this.processEvents(queryResults);
    }
    if (this.response.data.count > config.logfails) {
        this.response.result = results.FAILING;
    }
    
    return this.response;
}

HackerCheck.prototype.getQuery = function() {
    var twentyFourHoursAgo = new Date(new Date() - 24 * 60 * 60 * 1000);
    var twentyFourHoursAgoString = twentyFourHoursAgo.toISOString();
    // 2015-06-30T12:00:59.123Z
    // should be 20150630120059.123000-000, look at Microsoft documentation...
    twentyFourHoursAgoString = twentyFourHoursAgoString.split(/[^\d\.]/).join("") + "000-000";
    var query = "SELECT RecordNumber, TimeGenerated, EventCode, Type, Category, SourceName, Message, InsertionStrings FROM Win32_NTLogEvent WHERE Logfile='Security' and EventType=5 and (EventCode=4625 or EventCode=4768 or EventCode=4772 or EventCode=4771 or EventCode=4776 or EventCode=4777) AND TimeGenerated >= '" + twentyFourHoursAgoString + "'";
    return query;
}

HackerCheck.prototype.returnedEventsEmpty = function(queryResults) {
    if (Object.keys(queryResults).length == 0) {
        return true;
    }
    if (queryResults[0].RecordNumber === undefined) {
        return true;
    }
    return false;
}

HackerCheck.prototype.processEvents = function(queryResults) {
    var eventArray = [];
    for (var i = 0; i < Object.keys(queryResults).length; i++) {
        var eventCode = queryResults[i].EventCode;
        var event = this.getEventWithId(eventCode, eventArray);
        if (event) {
            this.updateEvent(queryResults[i], event);
        } else {
            eventArray.push(this.createEvent(queryResults[i]));
        }
    }
    return eventArray;
}

HackerCheck.prototype.getEventWithId = function(eventCode, eventArray) {
    for (var i = 0; i < eventArray.length; i++) {
        if (eventArray[i].event_id == eventCode) {
            return eventArray[i];
        }
    }
    return false;
}

HackerCheck.prototype.getSource = function(source, sourceArray) {
    for (var i = 0; i < sourceArray.length; i++) {
        if (sourceArray[i].value == source) {
            return sourceArray[i];
        }
    }
    return false;
}

HackerCheck.prototype.getUsername = function(username, usernameArray) {
    for (var i = 0; i < usernameArray.length; i++) {
        if (usernameArray[i].value == username) {
            return usernameArray[i];
        }
    }
    return false;
}

HackerCheck.prototype.updateEvent = function(eventQueryData, event) {
    event.count++;
    //Parsing these strings to js dates doesn't work in QtScript, using string compare.
    if (this.cimDateUtils.parseCimDateTimeUTCToIso(eventQueryData.TimeGenerated) < event.first_event) {
        event.first_event = this.cimDateUtils.parseCimDateTimeUTCToIso(eventQueryData.TimeGenerated);
    }
    if (this.cimDateUtils.parseCimDateTimeUTCToIso(eventQueryData.TimeGenerated) > event.last_event) {
        event.last_event = this.cimDateUtils.parseCimDateTimeUTCToIso(eventQueryData.TimeGenerated);
    }

    var source = this.getSourceFromInsertionString(eventQueryData.InsertionStrings, event.event_id);
    var sourceObject = this.getSource(source, event.event_detail.sources);
    if (sourceObject) {
        sourceObject.count++;
    } else {
        event.event_detail.sources.push(this.createSource(source));
    }

    var username = this.getUsernameFromInsertionString(eventQueryData.InsertionStrings, event.event_id);
    var usernameObject = this.getUsername(username, event.event_detail.usernames);
    if (usernameObject) {
        usernameObject.count++;
    } else {
        event.event_detail.usernames.push(this.createUsername(username));
    }
}

HackerCheck.prototype.createEvent = function(eventQueryData) {
    var event = {};
    event.event_id = eventQueryData.EventCode;
    event.count = 1;
    event.first_event = this.cimDateUtils.parseCimDateTimeUTCToIso(eventQueryData.TimeGenerated);
    event.last_event = this.cimDateUtils.parseCimDateTimeUTCToIso(eventQueryData.TimeGenerated);

    event.event_detail = {};
    var source = this.getSourceFromInsertionString(eventQueryData.InsertionStrings, event.event_id);
    event.event_detail.sources = [];
    event.event_detail.sources.push(this.createSource(source));

    var username = this.getUsernameFromInsertionString(eventQueryData.InsertionStrings, event.event_id);
    event.event_detail.usernames = [];
    event.event_detail.usernames.push(this.createUsername(username));

    return event;
}

HackerCheck.prototype.getSourceFromInsertionString = function(insertionStringsArray, eventID) {
    if (eventID == "4625") {
        return this.getNthInsertionString(insertionStringsArray, this.WMI_INSERTION_STRING_SOURCE_INDEX_FOR_4625);
    } else if (eventID == "4776") {
        return this.getNthInsertionString(insertionStringsArray, this.WMI_INSERTION_STRING_SOURCE_INDEX_FOR_4776);
    } else if (eventID == "4768") {
        return this.getNthInsertionString(insertionStringsArray, this.WMI_INSERTION_STRING_SOURCE_INDEX_FOR_4768);
    } else if (eventID == "4771") {
        return this.getNthInsertionString(insertionStringsArray, this.WMI_INSERTION_STRING_SOURCE_INDEX_FOR_4771);
    } else if (eventID == "4772") {
        return this.getNthInsertionString(insertionStringsArray, this.WMI_INSERTION_STRING_SOURCE_INDEX_FOR_4772);
    } else if (eventID == "4777") {
        return this.getNthInsertionString(insertionStringsArray, this.WMI_INSERTION_STRING_SOURCE_INDEX_FOR_4777);
    } else {
        return "-";
    }
}

HackerCheck.prototype.getUsernameFromInsertionString = function(insertionStringsArray, eventID) {
    if (eventID == "4625") {
        return this.getNthInsertionString(insertionStringsArray, this.WMI_INSERTION_STRING_USERNAME_INDEX_FOR_4625);
    } else if (eventID == "4776") {
        return this.getNthInsertionString(insertionStringsArray, this.WMI_INSERTION_STRING_USERNAME_INDEX_FOR_4776);
    } else if (eventID == "4768") {
        return this.getNthInsertionString(insertionStringsArray, this.WMI_INSERTION_STRING_USERNAME_INDEX_FOR_4768);
    } else if (eventID == "4771") {
        return this.getNthInsertionString(insertionStringsArray, this.WMI_INSERTION_STRING_USERNAME_INDEX_FOR_4771);
    } else if (eventID == "4772") {
        return this.getNthInsertionString(insertionStringsArray, this.WMI_INSERTION_STRING_USERNAME_INDEX_FOR_4772);
    } else if (eventID == "4777") {
        return this.getNthInsertionString(insertionStringsArray, this.WMI_INSERTION_STRING_USERNAME_INDEX_FOR_4777);
    } else {
        return "-";
    }
}

//Comes from WMI as an array, number passed in is 0 based
HackerCheck.prototype.getNthInsertionString = function(insertionStringsArray, number) {
    var element = insertionStringsArray[number];
    if(!element) return "-";
    var result = new String(element);
    return result.trim();
}

HackerCheck.prototype.createSource = function(source) {
    var sourceObject = {};
    sourceObject.value = source;
    sourceObject.count = 1;
    return sourceObject;
}

HackerCheck.prototype.createUsername = function(username) {
    var usernameObject = {};
    usernameObject.value = username;
    usernameObject.count = 1;
    return usernameObject;
}

function getCheck() {
    return new HackerCheck();
}/*CHECKSUM: 
79a55ec3f0f72bf7ea901a020db3bddeb496508e *C:\jenkins\workspace\FmPlugin_release_1.5.19\build\outputs\mainapp\RelWithDebInfo\Checks\check_hacker.js
*/ 
