'use strict';

// @importFmJsLib "Results.js"
// @importFmJsLib "Errors.js"

function PhysicalDiskSmartCheck() {
	this.SCSI_DRIVE_PATH = "\\\\.\\Scsi";
	this.MAX_SCSI_DRIVERS = 8;
	this.WMI_NAMESPACE = "\\\\.\\root\\cimv2";
	this.WMI_DISK_DRIVE_QUERY = "Select DeviceID, Status, SerialNumber from Win32_DiskDrive";
	this.WMI_OK_STATUS = "OK";
	this.CRITICAL_SMART_ATTRIBUTE_IDS = [5, 187, 188, 197, 198];


	this.needToRetrieveSmartDataThroughCsmi = false;

	this.response = {
		info: "",
		result: "",
		data: {}
	};
}

PhysicalDiskSmartCheck.prototype.run = function () {
	this.response.result = results.PASSING;
	this.response.data.smartDrives = [];

	var drives = this.getDriveListFromWmi();

	drives = this.fulfillDrivesWithSmartData(drives);

	this.determineCheckStatus(drives);
	this.pushDrivesIntoResponseData(drives);


	return this.response;
};

PhysicalDiskSmartCheck.prototype.getDriveListFromWmi = function () {
	var multipleDrivesSmartData = [];

	var apiCall = WMI_API.ConnectServer(this.WMI_NAMESPACE);
	if (!apiCall) {
		return this.fail(errors.ERROR_WMI_CANNOT_CONNECT);
	}

	var wmiDiskDriveQueryResults = WMI_API.ExecQuery(this.WMI_DISK_DRIVE_QUERY);
	var numberOfDisksFoundInWmi = Object.keys(wmiDiskDriveQueryResults).length;
	if (numberOfDisksFoundInWmi == 0) {
		return this.fail(errors.ERROR_WMI_CANNOT_EXECUTE_QUERY);
	}

	for (var i = 0; i < numberOfDisksFoundInWmi; i++) {
		var drive = {};
		drive.drivePath = wmiDiskDriveQueryResults[i].DeviceID;
		drive.wmiStatus = wmiDiskDriveQueryResults[i].Status;
		drive.serialNumber = wmiDiskDriveQueryResults[i].SerialNumber.trim();

		multipleDrivesSmartData.push(this.copyObject(drive));
	}

	return multipleDrivesSmartData;
};

PhysicalDiskSmartCheck.prototype.fulfillDrivesWithSmartData = function (drives) {
	drives = this.fulfillDrivesWithSmartDataFromPhysicalDriveSmartDataAPI(drives);
	if (this.needToRetrieveSmartDataThroughCsmi) {
		drives = this.fulfillDrivesWithSmartDataFromCsmiDriveSmartDataAPI(drives)
	}

	return drives;
};

PhysicalDiskSmartCheck.prototype.fulfillDrivesWithSmartDataFromPhysicalDriveSmartDataAPI = function (drives) {
	for (var i = 0; i < drives.length; i++) {
		var currentDrive = drives[i];
		var physicalDriveSmartData = this.getSmartDataForPhysicalDrive(currentDrive.drivePath);
		if (!!physicalDriveSmartData) {
			physicalDriveSmartData.serialNumber = currentDrive.serialNumber;
			physicalDriveSmartData.wmiStatus = currentDrive.wmiStatus;
			physicalDriveSmartData.drivePath = currentDrive.drivePath;
			drives[i] = physicalDriveSmartData;
		}
		else {
			this.needToRetrieveSmartDataThroughCsmi = true;
		}
	}
	return drives;
};

PhysicalDiskSmartCheck.prototype.getSmartDataForPhysicalDrive = function (drivePath) {
	var physicalDriveSmartData = null;
	if (PhysicalDriveSmartDataAPI.openDrive(drivePath)) {
		physicalDriveSmartData = PhysicalDriveSmartDataAPI.getSmartAttributesForDrive();
		PhysicalDriveSmartDataAPI.closeDrive();
	}
	if (!physicalDriveSmartData || Object.keys(physicalDriveSmartData).length == 0) {
		physicalDriveSmartData = null;
	}

	return physicalDriveSmartData;
};

PhysicalDiskSmartCheck.prototype.fulfillDrivesWithSmartDataFromCsmiDriveSmartDataAPI = function (drives) {
	var csmiDrives = this.getSmartDataForAllDrivesThroughCsmi();
	return this.mergeInSmartDataBySerialNumber(drives, csmiDrives);
};

PhysicalDiskSmartCheck.prototype.getSmartDataForAllDrivesThroughCsmi = function () {
	var csmiDrives = [];
	for (var i = 0; i < this.MAX_SCSI_DRIVERS; i++) {
		var csmiSmartDrivesAttachedToDriver = this.getSmartDataForDrivesAttachedToDriver(this.SCSI_DRIVE_PATH + i + ":");
		if (!!csmiSmartDrivesAttachedToDriver) {
			csmiDrives = csmiDrives.concat(csmiSmartDrivesAttachedToDriver);
		}
	}

	return csmiDrives;
};

PhysicalDiskSmartCheck.prototype.getSmartDataForDrivesAttachedToDriver = function (driverPath) {
	var csmiDrives = null;
	if (CsmiDriveSmartDataAPI.openScsiDriver(driverPath)) {
		csmiDrives = CsmiDriveSmartDataAPI.getSmartAttributesForDrivesAttachedToDriver();
		CsmiDriveSmartDataAPI.closeScsiDriver();
		if (!csmiDrives || csmiDrives.length == 0) {
			csmiDrives = null;
		}
	}

	return csmiDrives;
};

PhysicalDiskSmartCheck.prototype.mergeInSmartDataBySerialNumber = function (drives, drivesWithSmartData) {
	for (var i = 0; i < drives.length; i++) {
		var drive = drives[i];
		if (!(drive.smartAttributes)) {
			for (var j = 0; j < drivesWithSmartData.length; j++) {
				if (!!(drivesWithSmartData[j]) && !!(drivesWithSmartData[j].serialNumber) && (drive.serialNumber == drivesWithSmartData[j].serialNumber.trim())) {
					drive.smartAttributes = drivesWithSmartData[j].smartAttributes;
					drive.smartStatus = drivesWithSmartData[j].smartStatus;
					drive.warnings = drivesWithSmartData[j].warnings;
					break;
				}
			}
		}
		drives[i] = drive;
	}

	return drives;
};

PhysicalDiskSmartCheck.prototype.determineCheckStatus = function (drives) {
	var drive = null;
	for (var i = 0; i < drives.length; i++) {
		drive = drives[i];
		if (drive.wmiStatus != this.WMI_OK_STATUS) {
			this.fail(errors.ERROR_DRIVE_STATUS_NOT_OK);
			return;
		}
		if (!!drive.smartAttributes) {
			for (var j = 0; j < drive.smartAttributes.length; j++) {
				if (this.CRITICAL_SMART_ATTRIBUTE_IDS.indexOf(drive.smartAttributes[j].attributeId) != -1) {
					if (drive.smartAttributes[j].currentValue < drive.smartAttributes[j].threshold) {
						this.fail(errors.ERROR_DRIVE_STATUS_NOT_OK);
						return;
					}
				}
			}
		}
	}
};

PhysicalDiskSmartCheck.prototype.pushDrivesIntoResponseData = function (drives) {
	for (var i = 0; i < drives.length; i++) {
		this.response.data.smartDrives.push(this.copyObject(drives[i]));
	}
};

PhysicalDiskSmartCheck.prototype.copyObject = function (objectToBeCopied) {
	return JSON.parse(JSON.stringify(objectToBeCopied));
};

PhysicalDiskSmartCheck.prototype.fail = function (errCode) {
	if (this.response.result != results.FAILING) {
		this.response.data.error = errCode;
		this.response.result = results.FAILING;
	}
	return this.response;
};

function getCheck() {
	return new PhysicalDiskSmartCheck();
}
/*CHECKSUM: 
14dcfd3a5dc624711adfa24699216409f70a7200 *C:\jenkins\workspace\FmPlugin_release_1.5.19\build\outputs\mainapp\RelWithDebInfo\Checks\check_physical_disk_smart.js
*/ 
