'use strict';

// @importFmJsLib "Results.js"
// @importFmJsLib "Errors.js"

function SNMPCheck() {
    this.responses = [];

    this.OPERATOR_EQUAL = "0";
    this.OPERATOR_NOT_EQUAL = "1";
    this.OPERATOR_GREATER_THAN = "2";
    this.OPERATOR_LESS_THAN = "3";
    this.OPERATOR_GREATER_THAN_OR_EQUAL = "4";
    this.OPERATOR_LESS_THAN_OR_EQUAL = "5";

    this.sNMP_SYNTAX_INT = 2;
    this.sNMP_SYNTAX_BITS = 3;
    this.sNMP_SYNTAX_OCTETS = 4;
    this.sNMP_SYNTAX_NULL = 5;
    this.sNMP_SYNTAX_OID = 6;
    this.sNMP_SYNTAX_IPADDR = 64;
    this.sNMP_SYNTAX_CNTR32 = 65;
    this.sNMP_SYNTAX_GAUGE32 = 66;
    this.sNMP_SYNTAX_TIMETICKS = 67;
    this.sNMP_SYNTAX_OPAQUE = 68;
    this.sNMP_SYNTAX_CNTR64 = 70;
    this.sNMP_SYNTAX_NOSUCHOBJECT = 128;
    this.sNMP_SYNTAX_NOSUCHINSTANCE = 129;
    this.sNMP_SYNTAX_ENDOFMIBVIEW = 130;
    this.SNMP_CLASS_TIMEOUT = -5;

    this.TIME_ARRAY_SIZE = 3;
    this.HOURS_INDEX = 0;
    this.MINUTES_INDEX = 1;
    this.SECONDS_INDEX = 2;

    this.IP_ARRAY_SIZE = 4;
    this.IP_OCTECT_MAX_SIZE = 3;
    this.IP_OCTET_FILLER = 0;

    this.HOURS_TO_TIME_TICK_MULTIPLIER = 60 * 60 * 100;
    this.MINUTES_TO_TIME_TICK_MULTIPLIER = 60 * 100;
    this.SECONDS_TO_TIME_TICK_MULTIPLIER = 100;

    this.HEXADECIMAL_NULL_CHARACTER = "00";

    this.REG_EXP_HEX = "^[0-9A-Fa-f]{2}:[0-9A-Fa-f]{2}(:[0-9A-Fa-f]{2})*$"; // regular expression for hexadecimal strings (format: XX:XX:XX...)
    this.REG_EXP_OID = "^\\.?([0-9]+\\.)*[0-9]+$"; // regular expression for OIDs (we allow an OID to start with a dot)
    this.REG_EXP_HEX_DEC = "^0x[0-9A-Fa-f]+$"; // reg exp for hexadecimal strings starting with "0x"

    this.OIDValidator = new RegExp(this.REG_EXP_OID);
    this.HEXValidator = new RegExp(this.REG_EXP_HEX);
    this.HEXDecValidator = new RegExp(this.REG_EXP_HEX_DEC);
}

SNMPCheck.prototype.run = function(configList) {
    var oids = [];
    for (var configKey in configList) {
        var config = configList[configKey];
        var serverConfig = {
            host: config.host,
            port: config.port,
            community: config.community,
            version: config.snmpversion
        }; //assume all the same

        if (this.OIDValidator.test(config.oid)) {
            oids.push(config.oid);
        }
    }

    if (oids.length > 0) {
        var apiResponse = SNMPAPI.snmpRequest(oids, serverConfig);
        if (!apiResponse.success) {
            this.generateFailedResponses(configList, apiResponse.message);
        } else {
            this.generateSuccessfulResponses(apiResponse, configList, oids);
        }
    } else {
        this.generateInvalidOIDsResponses(configList);
    }

    return this.responses;
};

SNMPCheck.prototype.generateFailedResponses = function(configList, errorString) {
    for (var configKey in configList) {
        var config = configList[configKey];
        if (this.OIDValidator.test(config.oid)) {
            this.responses.push(this.generateSingleFailedResponse(configKey, errorString));
        } else {
            this.responses.push(this.generateSingleInvalidOIDResponse(configKey, config.oid));
        }
    }
};

SNMPCheck.prototype.generateSingleFailedResponse = function(checkId, errorMessage, errorCode) {
    if (!errorCode){
        errorCode = errors.ERROR_SNMP;
    }
    var response = {
        checkId: checkId,
        result: results.FAILING,
        data: {
            "errorCode": errorCode,
            "Output": errorMessage
        }
    };
    return response;
};

SNMPCheck.prototype.generateSingleInvalidOIDResponse = function(checkId, oid) {
    return this.generateSingleFailedResponse(checkId, "Invalid OID supplied: " + oid, errors.ERROR_SNMP_INVALID_OID);
}

SNMPCheck.prototype.generateInvalidOIDsResponses = function(configList) {
    for (var configKey in configList) {
        var config = configList[configKey];
        var response = this.generateSingleInvalidOIDResponse(configKey, config.oid);
        this.responses.push(response);
    }
}

SNMPCheck.prototype.generateSuccessfulResponses = function(apiResponse, configList, oidsRequested) {
    var oidResults = apiResponse.results;
    var oidResultsSyntax = apiResponse.resultsSyntax;
    for (var configKey in configList) {
        var config = configList[configKey];

        //If element in config list isn't in oids it must have been determined to be invalid
        // Therefore generate invalid OID response
        if (oidsRequested.indexOf(config.oid) == -1) {
            var response = this.generateSingleInvalidOIDResponse(configKey, config.oid);
        }
        else{
            var valueFromOid = oidResults[this.formatOidToMatchSnmpOutput(config.oid)];
            var syntaxFromOid = oidResultsSyntax[this.formatOidToMatchSnmpOutput(config.oid)];

            //If either the syntax or the value for the OID hasn't been defined the GET request has failed somehow
            // So set response as a failure other wise set as successful
            var response = (typeof syntaxFromOid == "undefined" || typeof valueFromOid == "undefined")
                    ? this.generateSingleFailedResponse(configKey, "SNMP GET Failed for OID: " + config.oid, errors.ERROR_SNMP_SNMPGET_FAILED)
                    : this.generateSingleSuccessfulResponse(valueFromOid, syntaxFromOid, config.op, config.testvalue, configKey, config.oid);

        }
      this.responses.push(response);
    }
};

SNMPCheck.prototype.formatOidToMatchSnmpOutput = function(oid) {
    //if you call SNMP lib with ".1.2.3" result comes back as "1.2.3"
    if (oid.indexOf('.') === 0) {
        return oid.substr(1);
    }
    return oid;
};

SNMPCheck.prototype.generateSingleSuccessfulResponse = function(output, syntax, operator, test_value, checkId, oid) {
    switch (syntax) {
        case this.sNMP_SYNTAX_TIMETICKS:
            return this.generateTimeTicksResponse(output, checkId, operator, test_value);
        case this.sNMP_SYNTAX_INT:
        case this.sNMP_SYNTAX_CNTR32:
        case this.sNMP_SYNTAX_GAUGE32:
        case this.sNMP_SYNTAX_CNTR64:
            return this.generateNumberResponse(output, checkId, operator, this.extractNumberIfAtStartOfString(test_value));
        case this.sNMP_SYNTAX_IPADDR:
            return this.generateIPResponse(output, checkId, operator, test_value);
        case this.sNMP_SYNTAX_OCTETS:
            return this.generateOctetResponse(output, checkId, operator, test_value);
        case this.sNMP_SYNTAX_OPAQUE:
            return this.generateSingleFailedResponse(checkId, "Unsupported SNMP variable type (OPAQUE)");
        case this.sNMP_SYNTAX_BITS:
            return this.generateSingleFailedResponse(checkId, "Unsupported SNMP variable type (BITS)");
        case this.sNMP_SYNTAX_NULL:
            return this.generateSingleFailedResponse(checkId, "Unsupported SNMP variable type (NULL)");
        case this.sNMP_SYNTAX_OID:
            return this.generateSingleFailedResponse(checkId, "Unsupported SNMP variable type (OID)");
        case this.sNMP_SYNTAX_NOSUCHOBJECT:
            return this.generateSingleFailedResponse(checkId, "No such object (" + oid + ")");
        case this.sNMP_SYNTAX_NOSUCHINSTANCE:
            return this.generateSingleFailedResponse(checkId, "No such object instance (" + oid + ")");
        case this.sNMP_SYNTAX_ENDOFMIBVIEW:
            return this.generateSingleFailedResponse(checkId, "Unknown object (" + oid + ")");
        case this.SNMP_CLASS_TIMEOUT:
            return this.generateSingleFailedResponse(checkId, "SNMP request timed out");
        default:
            return this.generateSingleFailedResponse(checkId, "Unsupported SNMP variable type (" + syntax  + ") returned for OID:" + oid);

    }
};

//if number return, if starts with a number return that
SNMPCheck.prototype.extractNumberIfAtStartOfString = function(input) {
    if (!isNaN(parseFloat(input))) {
        return parseFloat(input);
    }
    return input;
};

SNMPCheck.prototype.generateNumberResponse = function(output, checkId, operator, test_value) {

    //Some checks can return hexidecimal format starting with 0x. Detect these and parse them as numbers.
    if (this.HEXDecValidator.test(output)){
        // Explicit conversion from base 16 to base 10 to prevent incorrect implicit conversion in getCheckResult()
        var decimalOutput = parseInt(output, 16);
        return this.populateSuccesfulResponseData(
            decimalOutput,
            checkId,
            this.getCheckResult(decimalOutput, operator, test_value)
        );
    }
    // Not a Hexadecimal number so try to process as a number
    if (isNaN(test_value)) {
        return this.generateSingleFailedResponse(checkId, "User test value is invalid");
    } else {
        return this.populateSuccesfulResponseData(output, checkId, this.getCheckResult(parseFloat(output), operator, parseFloat(test_value)));
    }
}

SNMPCheck.prototype.generateTimeTicksResponse = function(output, checkId, operator, test_value) {
    var timeTicksFormattedResponse = this.getCorrectFormatForTimeTicks(test_value);
    if (!timeTicksFormattedResponse.success) {
        return this.generateSingleFailedResponse(checkId, "Invalid test data");
    }
    return this.populateSuccesfulResponseData(output, checkId, this.getCheckResult(parseFloat(output), operator, parseFloat(timeTicksFormattedResponse.value)));
}

SNMPCheck.prototype.generateOctetResponse = function(output, checkId, operator, test_value) {
    // first: check if the test_value has hexadecimal format
    if (this.HEXValidator.test(test_value)) {
        // looks like an hexadecima string!
        var formattedTestValue = test_value.toString().replace(/ /g, '').replace(/:/g, '');
        var formattedOutput = output.toUpperCase().replace(/(..)/g, "$1:").replace(/\:$/, "");
        return this.populateSuccesfulResponseData(formattedOutput, checkId, this.getCheckResult(output.toLowerCase(), operator, formattedTestValue.toLowerCase()));
    }


	if (this.endsWithNullChar(output)) {
        output = this.removeFinalNullCharacter(output);
    }
	
	var outputAsAscii = this.hex2Ascii(output);
	var outputAsNumber = this.extractNumberIfAtStartOfString(outputAsAscii);
	
	if(!isNaN(test_value) && !isNaN(outputAsNumber)) {
		// test value is a number, so we should compare with a number
		return this.populateSuccesfulResponseData(outputAsAscii, checkId, this.getCheckResult(outputAsNumber, operator, parseFloat(test_value)));
	}
    return this.populateSuccesfulResponseData(outputAsAscii, checkId, this.getCheckResult(output, operator, this.ascii2Hex(test_value)));
}

SNMPCheck.prototype.ascii2Hex = function(str) {
    var inputString = str.toString(); // force conversion for safety
    var arr = [];
    for (var i = 0, l = inputString.length; i < l; i++) {
        var hex = Number(inputString.charCodeAt(i)).toString(16);
        arr.push(hex);
    }
    return arr.join('');
}

SNMPCheck.prototype.endsWithNullChar = function(str) {
    return str.substr(str.length - 2, 2) == this.HEXADECIMAL_NULL_CHARACTER;
}

SNMPCheck.prototype.removeFinalNullCharacter = function(str) {
    return str.substr(0, str.length - 2);
}

SNMPCheck.prototype.hex2Ascii = function(hex) {
    var inputString = hex.toString(); // force conversion for safety
    var str = '';
    for (var i = 0; i < inputString.length; i += 2)
        str += String.fromCharCode(parseInt(inputString.substr(i, 2), 16));
    return str;
}

SNMPCheck.prototype.generateIPResponse = function(output, checkId, operator, test_value) {
    var testValueIPNumberFormat;

    if (isNaN(test_value)) {
        var arrayIPAddress = test_value.split(".");
        if (arrayIPAddress.length != 4) {
            return this.generateSingleFailedResponse(checkId, "Invalid test data");
        }
        testValueIPNumberFormat = this.IPToNumberFormat(arrayIPAddress);
    } else {
        // legacy, users could have entered IP as long int notation.
        testValueIPNumberFormat = test_value;
    }

    var arrayOutputIPAddress = output.split(".");
    var outputIPNumberFormat = this.IPToNumberFormat(arrayOutputIPAddress);

    return this.populateSuccesfulResponseData(output, checkId, this.getCheckResult(parseFloat(outputIPNumberFormat), operator, parseFloat(testValueIPNumberFormat)));
}

SNMPCheck.prototype.IPToNumberFormat = function(IPAddressArray) {
    var formattedIPArray = [];
    formattedIPArray.push(IPAddressArray[0]);
    for (var i = 1; i < this.IP_ARRAY_SIZE; i++) {
        formattedIPArray.push(this.leftPad(IPAddressArray[i], this.IP_OCTET_FILLER, this.IP_OCTECT_MAX_SIZE));
    }
    return formattedIPArray.join('');
}

SNMPCheck.prototype.leftPad = function(str, character, size) {
    // prepend to the string str the character until the string length is equals to size
    return Array(Math.max(size - String(str).length + 1, 0)).join(character) + str;
}

SNMPCheck.prototype.getCorrectFormatForTimeTicks = function(test_value) {
    var result = {
        success: false,
        value: 0
    };

    if (!isNaN(test_value)) {
        result.success = true;
        result.value = parseFloat(test_value);
    } else if (~test_value.indexOf(':')) {
        result = this.stringToTimeTicksFormat(test_value);
    }
    return result;
}

SNMPCheck.prototype.stringToTimeTicksFormat = function(timeString) {
    // Supported format is [...]x:yy:zz.ab
    // Where leading numeric x is int hours
    //                      yy is (int) minutes
    //                      zz is (int) seconds
    //                      ab is (int) tenths/hundreths of second
    var result = {
        success: false,
        value: 0
    }
    var timeArray = timeString.split(":");

    if (timeArray.length < this.TIME_ARRAY_SIZE ||
        isNaN(timeArray[this.HOURS_INDEX]) ||
        isNaN(timeArray[this.MINUTES_INDEX]) ||
        isNaN(timeArray[this.SECONDS_INDEX])) {
        return result;
    }

    var hoursInTimeTicksFormat = parseFloat(timeArray[this.HOURS_INDEX]) * this.HOURS_TO_TIME_TICK_MULTIPLIER;
    var minutesInTimeTicksFormat = parseFloat(timeArray[this.MINUTES_INDEX]) * this.MINUTES_TO_TIME_TICK_MULTIPLIER;
    var secondsInTimeTicksFormat = parseFloat(timeArray[this.SECONDS_INDEX]) * this.SECONDS_TO_TIME_TICK_MULTIPLIER;

    result.value = hoursInTimeTicksFormat + minutesInTimeTicksFormat + secondsInTimeTicksFormat;
    result.success = true;

    return result;
}

SNMPCheck.prototype.getCheckResult = function(output, operator, test_value) {
    switch (operator) {
        case this.OPERATOR_EQUAL:
            return output == test_value;
        case this.OPERATOR_GREATER_THAN:
            return output > test_value;
        case this.OPERATOR_LESS_THAN:
            return output < test_value;
        case this.OPERATOR_GREATER_THAN_OR_EQUAL:
            return output >= test_value;
        case this.OPERATOR_LESS_THAN_OR_EQUAL:
            return output <= test_value;
        case this.OPERATOR_NOT_EQUAL:
            return output != test_value;
    }
    return false;
}

SNMPCheck.prototype.populateSuccesfulResponseData = function(output, checkId, result) {
    var checkResult = result ? results.PASSING : results.FAILING;
    var response = {
        checkId: checkId,
        result: checkResult,
        data: {
            "Output": output
        }
    };
    return response;
}

function getCheck() {
    return new SNMPCheck();
}
/*CHECKSUM: 
72888c26273506cd46576e675578aed5693af8fe *C:\jenkins\workspace\FmPlugin_release_1.5.19\build\outputs\mainapp\RelWithDebInfo\Checks\check_snmp.js
*/ 
